﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.MainForm;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.DeploymentService.AdminClient {
  public partial class PluginListView : HeuristicLab.MainForm.WindowsForms.View {
    private List<ListViewItem> localPlugins;
    private List<ListViewItem> serverPlugins;
    private BackgroundWorker pluginUploadWorker;

    public PluginListView() {
      InitializeComponent();
      Caption = "Plugins";

      localPlugins = new List<ListViewItem>();
      serverPlugins = new List<ListViewItem>();

      pluginUploadWorker = new BackgroundWorker();
      pluginUploadWorker.DoWork += new DoWorkEventHandler(pluginUploadWorker_DoWork);
      pluginUploadWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(pluginUploadWorker_RunWorkerCompleted);

      UpdateLocalPlugins();
    }

    void pluginUploadWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
      this.Enabled = true;
    }

    void pluginUploadWorker_DoWork(object sender, DoWorkEventArgs e) {
      var selectedPlugins = (IEnumerable<IPluginDescription>)e.Argument;
      AdminService.AdminClient adminClient = new HeuristicLab.DeploymentService.AdminClient.AdminService.AdminClient();

      foreach (var plugin in IteratePlugins(selectedPlugins)) {
        adminClient.DeployPlugin(MakePluginDescription(plugin), new byte[0]);
      }
    }

    private HeuristicLab.PluginInfrastructure.PluginUpdateService.PluginDescription MakePluginDescription(IPluginDescription plugin) {
      var dependencies = from dep in plugin.Dependencies
                         select MakePluginDescription(dep);
      return new HeuristicLab.PluginInfrastructure.PluginUpdateService.PluginDescription(plugin.Name, plugin.Version, dependencies);
    }

    private IEnumerable<IPluginDescription> IteratePlugins(IEnumerable<IPluginDescription> plugins) {
      foreach (var plugin in plugins) {
        foreach (var dependency in IteratePlugins(plugin.Dependencies)) {
          yield return dependency;
        }
        yield return plugin;
      }
    }

    public void UpdateLocalPlugins() {
      localPlugins.ForEach(x => listView.Items.Remove(x));
      localPlugins.Clear();
      foreach (var plugin in ApplicationManager.Manager.Plugins) {
        var item = MakeListViewItem(plugin);
        item.Group = listView.Groups["Local plugins"];
        listView.Items.Add(item);
        localPlugins.Add(item);
      }
    }

    public void UpdateServerPlugins() {
      serverPlugins.ForEach(x => listView.Items.Remove(x));
      serverPlugins.Clear();
      var client = new HeuristicLab.PluginInfrastructure.PluginUpdateService.UpdateClient();
      foreach (var plugin in client.GetPlugins()) {
        var item = MakeListViewItem(plugin);
        item.Group = listView.Groups["Server plugins"];
        listView.Items.Add(item);
        localPlugins.Add(item);
      }
    }

    private static ListViewItem MakeListViewItem(HeuristicLab.PluginInfrastructure.PluginUpdateService.PluginDescription plugin) {
      ListViewItem item = new ListViewItem(new string[] { plugin.Name, plugin.Version.ToString() });
      item.Tag = plugin;
      return item;
    }

    private static ListViewItem MakeListViewItem(IPluginDescription plugin) {
      ListViewItem item = new ListViewItem(new string[] { plugin.Name, plugin.Version.ToString() });
      item.Tag = plugin;
      return item;
    }

    private void listView_SelectedIndexChanged(object sender, EventArgs e) {
      if (listView.SelectedItems.Count > 0) {
        if (listView.SelectedItems[0].Tag is IPluginDescription) {
          var plugin = (IPluginDescription)listView.SelectedItems[0].Tag;
          StringBuilder strBuilder = new StringBuilder();

          strBuilder.Append("Name: ").AppendLine(plugin.Name);
          strBuilder.Append("Version: ").AppendLine(plugin.Version.ToString());
          strBuilder.AppendLine("Files:");
          foreach (var file in plugin.Files) {
            strBuilder.Append(file.Name + " " + file.Type);
          }
          strBuilder.AppendLine("Dependencies:");
          foreach (var dep in plugin.Dependencies) {
            strBuilder.Append(dep.Name + " " + dep.Version);
          }
          detailsTextBox.Text = strBuilder.ToString();
        } else if (listView.SelectedItems[0].Tag is HeuristicLab.PluginInfrastructure.PluginUpdateService.PluginDescription) {
          var plugin = (HeuristicLab.PluginInfrastructure.PluginUpdateService.PluginDescription)listView.SelectedItems[0].Tag;
          StringBuilder strBuilder = new StringBuilder();

          strBuilder.Append("Name: ").AppendLine(plugin.Name);
          strBuilder.Append("Version: ").AppendLine(plugin.Version.ToString());
          strBuilder.AppendLine("Dependencies:");
          foreach (var dep in plugin.Dependencies) {
            strBuilder.Append(dep.Name + " " + dep.Version);
          }
          detailsTextBox.Text = strBuilder.ToString();
        }
      } else {
        detailsTextBox.Text = string.Empty;
      }
    }

    private void uploadButton_Click(object sender, EventArgs e) {
      if (listView.SelectedItems.Count > 0) {
        var selectedPlugins = from item in listView.SelectedItems.Cast<ListViewItem>()
                              where item.Tag is IPluginDescription
                              select item.Tag as IPluginDescription;
        this.Enabled = false;
        pluginUploadWorker.RunWorkerAsync(selectedPlugins.ToList());
      }
    }
  }
}
